@@ -47,8 +47,8 @@ setGeneric("pmapFromTranscripts", signature=c("x", "transcripts"),
     pend <- end(part)[width(part) != 0L]
 
     neg <- strand(gr)[pstart] == "-"
-    ord <- S4Vectors:::mseq(ifelse(neg, pend, pstart),
-                            ifelse(neg, pstart, pend))
+    ord <- sequence(width(part), from=ifelse(neg, pend, pstart),
+                                 by=ifelse(neg, -1L, 1L))
     res <- relist(gr[ord], x)
     res@unlistData$unordered <- original[idx[ord]]
     res
@@ -224,7 +224,7 @@ setMethod("mapToTranscripts", c("ANY", "TxDb"),
         if (!is.function(extractor.fun))
             stop("'extractor.fun' must be a function")
         group1 <- c("transcripts", "exons", "cds", "genes", "promoters",
-                    "disjointExons", "microRNAs", "tRNAs")
+                    "exonicParts", "microRNAs", "tRNAs")
         group2 <- c("transcriptsBy", "exonsBy", "cdsBy", "intronsByTranscript",
                    "fiveUTRsByTranscript", "threeUTRsByTranscript")
 
